<?php

declare(strict_types=1);

namespace Imi\Db\Mysql\Query\Builder;

use Imi\Db\Query\Field;

abstract class BaseBuilder extends \Imi\Db\Query\Builder\BaseBuilder
{
    /**
     * {@inheritDoc}
     */
    protected function parseField(array $fields): string
    {
        if (!$fields)
        {
            return '*';
        }
        $result = [];
        $query = $this->query;
        $params = &$this->params;
        foreach ($fields as $k => $v)
        {
            if (is_numeric($k))
            {
                if ($v instanceof Field)
                {
                    $field = $v;
                }
                else
                {
                    $field = new Field();
                    $field->setValue($v ?? '', $query);
                }
            }
            else
            {
                $field = new Field(null, null, $k, $v);
            }
            $result[] = $field->toString($query);
            $binds = $field->getBinds();
            if ($binds)
            {
                $params = array_merge($params, $binds);
            }
        }

        return implode(',', $result);
    }
}
